@@ -25,7 +25,7 @@
  */
 
 #ident "@(#) Copyright (C)2002..2022 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: cmd_copy.c,v 1.85 2022/01/08 22:59:07 babolo Exp $"
+#ident "@(#) $Id: cmd_copy.c,v 1.86 2022/03/26 22:17:10 babolo Exp $"
 
 #define BLIN_COMPAT      4
 #define Bpars_COMPAT     3
@@ -101,11 +101,11 @@ pgoblin_copyout(pgoblin_exenv *exenv, pgoblin_nr *rn) {             /**
             ifBLIN_QX2("%d buf=%s", f, (char*)buf);
             if  (strcmp("\\.", buf) == 0) {
                 if  (exenv->options->flags & PGOBLIN_COPEND_PASS)
-                    e = rou->wri(rou->onu, buf, f);
+                    e = pgoblin_wr(rou, buf, f);
                 free(buf);
                 break;
             }
-            e = rou->wri(rou->onu, buf, f);
+            e = pgoblin_wr(rou, buf, f);
             free(buf);
         }
         if  ((e < 0) || (f < 0)) {
@@ -122,7 +122,7 @@ pgoblin_copyout(pgoblin_exenv *exenv, pgoblin_nr *rn) {             /**
     if  (e < 0) {
         ifBLIN_QX0("write error");
     } else if (exenv->options->flags & PGOBLIN_COPEND_APND) {
-        e = rou->wri(rou->onu, "\\.\n", 0);
+        e = pgoblin_wr(rou, "\\.\n", 0);
     }
     if  (e < 0) {
         ifBLIN_QX0("write end error");
@@ -423,7 +423,7 @@ pgoblin_getmeta(pgoblin_exenv *exenv, pgoblin_nr *rn) {             /**
         }
         l -= e;
         k += e;
-        if  (0 > (e = rou->wri(rou->onu, buf, k))) {
+        if  (0 > (e = pgoblin_wr(rou, buf, k))) {
             ifBLIN_QW0("write error");
             free(buf);
             goto out;
@@ -431,7 +431,7 @@ pgoblin_getmeta(pgoblin_exenv *exenv, pgoblin_nr *rn) {             /**
         free(buf);
     }
     if  (exenv->options->flags & PGOBLIN_COPEND_APND) {
-        if  (0 > (e = rou->wri(rou->onu, "\\.\n", 3))) ifBLIN_QW0("write end error");
+        if  (0 > (e = pgoblin_wr(rou, "\\.\n", 3))) ifBLIN_QW0("write end error");
     }
 out:
     ifBLIN_QX3("- %"BLIN_D, e);
