@@ -25,7 +25,7 @@
  */
 
 #ident "@(#) Copyright (C)2002..2022 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: cmd_select.c,v 1.112 2022/01/15 17:11:23 babolo Exp $"
+#ident "@(#) $Id: cmd_select.c,v 1.113 2022/05/01 20:40:19 babolo Exp $"
 
 #define BLIN_COMPAT      4
 #define Bpars_COMPAT     3
@@ -96,15 +96,18 @@ presel(pgoblin_exenv *exenv, pgoblin_nr *rn, BLIN_flag cond) {      /**
     ;;  rdb->flags |= cond;
     MARK_R_CONN_WENT(rn[PGO_CCON]);;
     if  ((rct->flags & PGOBLIN_PQRESULT) && !!rct->pq) {
-        /*   CTL */
-        ifBLIN_QX5("CTLNtuples=%d", pgoblin_db_resinfo(exenv, rn[PGO_CCTL], PGOBLIN_Ntuples));
-        for (ssize_t k = 0; k < pgoblin_db_resinfo(exenv, rn[PGO_CCTL], PGOBLIN_Ntuples); ++k) {
+        ssize_t ctupc; /*   CTL  ctupc */
+
+        ctupc = pgoblin_db_resinfo(exenv, rn[PGO_CCTL], PGOBLIN_Ntuples);
+        ifBLIN_QX5("CTLNtuples=%d", ctupc);
+        for (ssize_t ctupn = 0; ctupn < ctupc; ++ctupn) {
             MARK_IO_TEXT_GO(rn[PGO_CCTL]);;
         ;   ;;  if  (!!rct->text && (rct->flags & PGOBLIN_FREETEXT)) {
         ;   ;;      free(rct->text);
         ;   ;;      rct->text = NULL;
         ;   ;;  }
-        ;   ;;  if  (0 > (ex = pgoblin_style_0.table(exenv, 0, rn[PGO_CCTL], rn[PGO_CCTL], k, -3))) {
+        ;   ;;  ex = pgoblin_style_0.table(exenv, 0, rn[PGO_CCTL], rn[PGO_CCTL], ctupn, -3);
+        ;   ;;  if  (0 > ex) {
         ;   ;;      ifBLIN_QW0("No STYLE 0");
         ;   ;;      goto out;
         ;   ;;  }
@@ -148,30 +151,6 @@ presel(pgoblin_exenv *exenv, pgoblin_nr *rn, BLIN_flag cond) {      /**
                  /* ;; */                     , PGOBLIN_REGSIZE - 1
                  /* ;; */                     , rn[PGO_CIN]
                  /* ;; */                     );
-        ;           MARK_IO_PQ_WENT(rn[PGO_COUT]);;
-        ;           if  (!!ex) {
-        ;               ifBLIN_QX0( "%s: execute Q=%s"
-                                  , pgoblin_decond(cond)
-                                  , pgoblin_db_erinfo(exenv, rn[PGO_COUT])
-                                  );
-        ;               MARK_IO_PQ_GO(rn[PGO_COUT]);;
-        ;               ;;  pgoblin_db_clear(exenv, rn[PGO_COUT]);
-        ;               MARK_IO_PQ_WENT(rn[PGO_COUT]);;
-        ;               ERROUT(EX_SOFTWARE, EDOOFUS);
-        ;           }
-        ;           if  (  (k + 1 < pgoblin_db_resinfo(exenv, rn[PGO_CCTL], PGOBLIN_Ntuples))
-                        || ((PGOBLIN_CR_OK == cond) && !rn[PGO_COUT])
-                        ) {
-        ;               pgoblin_db_clear(exenv, rn[PGO_COUT]);
-        ;           } else {
-        ;               MARK_IO_PQ_GO(rn[PGO_COUT]);;
-        ;               ;;  if  (rou->pq) rou->flags |= PGOBLIN_PQRESULT | PGOBLIN_FREEPQRE;
-        ;               ;;  rou->cortege = -1;
-        ;               MARK_IO_PQ_WENT(rn[PGO_COUT]);;
-        ;           }
-        ;           MARK_IO_PQ_GO(rn[PGO_COUT]);;
-        ;           ;;  pgoblin_db_clear(exenv, rn[PGO_COUT]);
-        ;           MARK_IO_PQ_WENT(rn[PGO_COUT]);;
         ;       } else {
         ;           MARK_IO_PQ_GO(rn[PGO_COUT]);;
         ;           ;;  ex = pgoblin_db_execute( exenv
@@ -180,29 +159,31 @@ presel(pgoblin_exenv *exenv, pgoblin_nr *rn, BLIN_flag cond) {      /**
                  /* ;; */                      , rn[PGO_CCTL]
                  /* ;; */                      , rn[PGO_CIN]
                  /* ;; */                      );
+        ;       }
+        ;       MARK_IO_PQ_WENT(rn[PGO_COUT]);;
+        ;       if  (!!ex) {
+        ;           ifBLIN_QX0( "%s: execute Q=%s"
+                              , pgoblin_decond(cond)
+                              , pgoblin_db_erinfo(exenv, rn[PGO_COUT])
+                              );
+        ;           MARK_IO_PQ_GO(rn[PGO_COUT]);;
+        ;           ;;  pgoblin_db_clear(exenv, rn[PGO_COUT]);
         ;           MARK_IO_PQ_WENT(rn[PGO_COUT]);;
-        ;           if  (!!ex) {
-        ;               ifBLIN_QX0( "%s: execute Q=%s"
-                                  , pgoblin_decond(cond)
-                                  , pgoblin_db_erinfo(exenv, rn[PGO_COUT])
-                                  );
-        ;               MARK_IO_PQ_GO(rn[PGO_COUT]);;
-        ;               ;;  pgoblin_db_clear(exenv, rn[PGO_COUT]);
-        ;               MARK_IO_PQ_WENT(rn[PGO_COUT]);;
-        ;               ERROUT(EX_SOFTWARE, EDOOFUS);
-        ;           }
-        ;           if  (  (k + 1 < pgoblin_db_resinfo(exenv, rn[PGO_CCTL], PGOBLIN_Ntuples))
-                        || ((PGOBLIN_CR_OK == cond) && !rn[PGO_COUT])
-                        ) {
-        ;               MARK_IO_PQ_GO(rn[PGO_COUT]);;
-        ;               ;;  pgoblin_db_clear(exenv, rn[PGO_COUT]);
-        ;               MARK_IO_PQ_WENT(rn[PGO_COUT]);;
-        ;           } else {
-        ;               MARK_IO_PQ_GO(rn[PGO_COUT]);;
-        ;               ;;  if  (rou->pq) rou->flags |= PGOBLIN_PQRESULT | PGOBLIN_FREEPQRE;
-        ;               ;;  rou->cortege = -1;
-        ;               MARK_IO_PQ_WENT(rn[PGO_COUT]);;
-        ;       }   }
+        ;           ERROUT(EX_SOFTWARE, EDOOFUS);
+        ;       }
+        ;       ifBLIN_QX6("ctupn %"BLIN_D" to ctupc %"BLIN_D, ctupn, ctupc);
+        ;       if  (  (ctupn + 1 < ctupc)
+                    || ((PGOBLIN_CR_OK == cond) && !rn[PGO_COUT])
+                    ) {
+        ;           MARK_IO_PQ_GO(rn[PGO_COUT]);;
+        ;           pgoblin_db_clear(exenv, rn[PGO_COUT]);
+        ;           MARK_IO_PQ_WENT(rn[PGO_COUT]);;
+        ;       } else {
+        ;           MARK_IO_PQ_GO(rn[PGO_COUT]);;
+        ;           ;;  if  (rou->pq) rou->flags |= PGOBLIN_PQRESULT | PGOBLIN_FREEPQRE;
+        ;           ;;  rou->cortege = -1;
+        ;           MARK_IO_PQ_WENT(rn[PGO_COUT]);;
+        ;       }
         ;   } else {
         ;       if  ((ex = pgoblin_db_query(exenv, rn[PGO_CCON], rn[PGO_COUT], rn[PGO_CCTL]))) {
         ;           ifBLIN_QX0( "%s: query=%s"
@@ -214,7 +195,7 @@ presel(pgoblin_exenv *exenv, pgoblin_nr *rn, BLIN_flag cond) {      /**
         ;           MARK_IO_PQ_WENT(rn[PGO_COUT]);;
         ;           ERROUT(EX_SOFTWARE, EDOOFUS);
         ;       }
-        ;       if  (  (k + 1 < pgoblin_db_resinfo(exenv, rn[PGO_CCTL], PGOBLIN_Ntuples))
+        ;       if  (  (ctupn + 1 < ctupc)
                     || ((PGOBLIN_CR_OK == cond) && !rn[PGO_COUT])
                     ) {
         ;           MARK_IO_PQ_GO(rn[PGO_COUT]);;
