@@ -25,7 +25,7 @@
  */
 
 #ident "@(#) Copyright (C)2002..2022 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: cmd_trans.c,v 1.149 2022/01/23 16:13:38 babolo Exp $"
+#ident "@(#) $Id: cmd_trans.c,v 1.151 2022/03/27 16:29:48 babolo Exp $"
 
 #define BLIN_COMPAT      4
 #define Bpars_COMPAT     3
@@ -766,9 +766,7 @@ pgoblin_listen(pgoblin_exenv *exenv, pgoblin_nr *rn) {              /**
     GET_RIO(rct, exenv->options, rn[PGO_CCTL]);
     GET_RIO(rin, exenv->options, rn[PGO_CIN]);
     if  (!rin->text || (rin->length < sizeof(int64_t))) {
-        ifBLIN_QX0(" IN[%d]: no timeout", rn[PGO_CIN]);
-        ex = EX_DATAERR;
-        errno = EINVAL;
+        ifBLIN_QX1(" IN[%d]: no timeout", rn[PGO_CIN]);
     } else {
         /* Should be 64 bit time wait to in usec from 1970-01-01 00:00:00 UTC          *
          * (date_part('epoch', t) * 1000000)::int8                                     *
@@ -786,9 +784,7 @@ pgoblin_listen(pgoblin_exenv *exenv, pgoblin_nr *rn) {              /**
         ;   now = (int64_t)1000000LL * t.tv_sec + t.tv_usec;
         ;   timeout = (itime - now + 999) / 1000;
         ;   if  ((timeout) <= 0) {
-        ;       ifBLIN_QX0("tarry timeout");
-        ;       ex = EX_TEMPFAIL;
-        ;       errno = ETIMEDOUT;
+        ;       ifBLIN_QX0("tarry notime");
         ;       break;
         ;   } else if (timeout > exenv->options->maxtmout) {
         ;       timeout = exenv->options->maxtmout;
@@ -798,19 +794,15 @@ pgoblin_listen(pgoblin_exenv *exenv, pgoblin_nr *rn) {              /**
         ;   ifBLIN_QX3("LISTEN=%s~ for %d ms", (char*)rct->text, (int)timeout);
         ;   e = db->tarry(exenv, rn[PGO_CCON], (int)timeout);
         ;   if  (0 > e) {
-        ;       if  (!db->erconn) {
-        ;           ifBLIN_QX0("tarry error");
-        ;       } else {
-        ;           ifBLIN_QX0("tarry error: %s", db->erconn(exenv, rn[PGO_CCON]));
-        ;       }
-        ;   } else if (1 < e) {
         ;       ifBLIN_QW0("tarry error");
         ;       ex = e;
         ;       break;
         ;   } else if (!e) {
-        ;       ifBLIN_QX0("tarry timeout");
-        ;       ex = EX_TEMPFAIL;
-        ;       errno = ETIMEDOUT;
+        ;       if  (!db->erconn) {
+        ;           ifBLIN_QX1("tarry timeout");
+        ;       } else {
+        ;           ifBLIN_QX1("tarry timeout: %s", db->erconn(exenv, rn[PGO_CCON]));
+        ;       }
         ;       break;
         ;   }
         ;   if  (PGOBLIN_LONGNOTIFY & db->flags) {
