@@ -25,7 +25,7 @@
  */
 
 #ident "@(#) Copyright (C)2017..2022 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: db.c,v 1.53 2022/01/23 14:19:59 babolo Exp $"
+#ident "@(#) $Id: db.c,v 1.54 2022/03/20 19:11:52 babolo Exp $"
 
 #define BLIN_COMPAT      4
 #define Bpars_COMPAT     3
@@ -178,6 +178,10 @@ pgoblin_db_resinfo(pgoblin_exenv *exenv, pgoblin_nr nio, int kind) { /**
         ifBLIN_QX1("No DB or resinfo");
         errno = ENOSYS;
         ex = -EX_UNAVAILABLE;
+    } else if (  !!(rio->flags & PGOBLIN_PURGE)
+              && ((PGOBLIN_Ntuples == kind) || (PGOBLIN_Nfields == kind))
+              ) {
+        ex = 0;
     } else if (  (1 < (ex = DBASE(*rio->pq)->resinfo(exenv, nio, kind)))
               && (PGOBLIN_Ntuples == kind)
               && (0 <= rio->cortege)
@@ -208,6 +212,10 @@ pgoblin_db_subinfo(pgoblin_exenv *exenv, pgoblin_nr nio, int kind, int64_t sub)
         errno = EFAULT;
     } else if (!DBASE(*rio->pq) || !DBASE(*rio->pq)->subinfo) {
         errno = ENOSYS;
+    } else if (  !!(rio->flags & PGOBLIN_PURGE)
+              && ((PGOBLIN_TypeNm == kind) || (PGOBLIN_ColName == kind))
+              ) {
+        errno = ENOENT;
     } else {
         c = DBASE(*rio->pq)->subinfo(exenv, nio, kind, sub);
     }
@@ -262,6 +270,8 @@ pgoblin_db_valinfo(pgoblin_exenv *exenv, pgoblin_nr nio, int kind, int row, int
         errno = EFAULT;
     } else if (!DBASE(*rio->pq) || !DBASE(*rio->pq)->valinfo) {
         errno = ENOSYS;
+    } else if (!!(rio->flags & PGOBLIN_PURGE) && (PGOBLIN_Length == kind)) {
+        ex = 0;
     } else {
         ex = DBASE(*rio->pq)->valinfo(exenv, nio, kind, (0 > rio->cortege) ? row : rio->cortege, col);
     }
@@ -294,6 +304,8 @@ pgoblin_db_getvalue(pgoblin_exenv *exenv, pgoblin_nr nio, int row, int col) { /*
         errno = EFAULT;
     } else if (!DBASE(*(rio->pq)) || !DBASE(*(rio->pq))->getvalue) {
         errno = ENOSYS;
+    } else if (!!(rio->flags & PGOBLIN_PURGE)) {
+        errno = ENOENT;
     } else {
         c = DBASE(*(rio->pq))->getvalue(exenv, nio, (0 > rio->cortege) ? row : rio->cortege, col);
     }
