@@ -25,7 +25,7 @@
  */
 
 #ident "@(#) Copyright (C)2007..2022 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: descript.c,v 1.97 2022/01/23 14:19:59 babolo Exp $"
+#ident "@(#) $Id: descript.c,v 1.98 2022/03/20 19:11:52 babolo Exp $"
 
 #define BLIN_COMPAT      4
 #define Bpars_COMPAT     3
@@ -63,6 +63,7 @@
 #define Fu PGOBLIN_LEXLVUP
 #define Fd PGOBLIN_LEXLDOWN
 #define Fc PGOBLIN_CTLCORTG
+#define Fq PGOBLIN_PURGE
 #define Io PGOBLIN_OUTSET
 #define Id PGOBLIN_CTL
 #define Ii PGOBLIN_ISIN
@@ -493,7 +494,7 @@ const pgoblin_syntax pgoblin =
   , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
   , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
 
-  , {    Io|Id|Ii|Ic          , {Po|Pq, Pt|Pq,    Pc,   Pq}, pgoblin_perform, "perform"  } /* 020 */
+  , { Fq|Io|Id|Ii|Ic          , {Po|Pq, Pt|Pq,    Pc,   Pq}, pgoblin_perform, "perform"  } /* 020 */
 //  , 
 //a
 // CONN
