@@ -25,7 +25,7 @@
  */
 
 #ident "@(#) Copyright (C)2002..2022 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: execute.c,v 1.216 2022/01/23 18:06:14 babolo Exp $"
+#ident "@(#) $Id: execute.c,v 1.220 2022/04/10 17:35:44 babolo Exp $"
 
 #define BLIN_COMPAT      4
 #define Bpars_COMPAT     3
@@ -70,7 +70,7 @@
         buflen = 0;                                   \
 }   } while(0)
 
-BLIN_flag pgoblin_default;
+extern BLIN_flag pgoblin_default;
 static size_t u12[] = {341, 512, 512};
 
 static void
@@ -360,7 +360,7 @@ cleareg(pgoblin_exenv *exenv, pgoblin_nr r, u_int32_t m) {          /**
         ;;      pgoblin_db_clear(exenv, r);
         ;;  }
         ;;  rio->pq = NULL;
-        ;;  rio->flags &= ~(PGOBLIN_FREEPQRE | PGOBLIN_PQRESULT);
+        ;;  rio->flags &= ~(PGOBLIN_FREEPQRE | PGOBLIN_PQRESULT | PGOBLIN_PURGE);
         ;;  rio->cortege = -1;
         MARK_IO_PQ_WENT(r);;
     }
@@ -384,7 +384,7 @@ cleareg(pgoblin_exenv *exenv, pgoblin_nr r, u_int32_t m) {          /**
         ;;  rio->clo = NULL;
         ;;  rio->wri = NULL;
         ;;  rio->onu = 0;
-        ;;  rio->flags &= ~(PGOBLIN_OUTSET | PGOBLIN_FREEOUTS);
+        ;;  rio->flags &= ~(PGOBLIN_OUTSET | PGOBLIN_FREEOUTS | PGOBLIN_OUTCLEAR);
         MARK_IO_OUT_WENT(r);;
     }
     if  (!!cl) {
@@ -721,7 +721,14 @@ coercion(pgoblin_exenv *exenv, pgoblin_nr r, u_int32_t ch) {        /**
               || ((rio->flags & PGOBLIN_MIFEDES ) && !rio->mife)
               || ((rio->flags & PGOBLIN_PQRESULT) && !rio->pq  )
               ) {
-        ifBLIN_QX0("No info f=%08X", rio->flags);
+        ifBLIN_QX0( "No info for IO=%c f=%08X%s%s%s%s"
+                  , pgoblin_regn[r]
+                  , rio->flags
+                  , (rio->flags & PGOBLIN_PATHPARM) ? " path" : ""
+                  , (rio->flags & PGOBLIN_TEXTPARM) ? " text" : ""
+                  , (rio->flags & PGOBLIN_MIFEDES ) ? " mife" : ""
+                  , (rio->flags & PGOBLIN_PQRESULT) ? " pq" : ""
+                  );
         ERROUT(EX_DATAERR, EINVAL);
     }
     price = coercomp(exenv->options, rio, ch);
@@ -1330,7 +1337,9 @@ pgoblin_execute( pgoblin_main  *options      /**/
     ;           CICLEWARN(0, "No propush");
     ;       } else if ((RNOUT == RNCTL) || (RNOUT == RNIN)) {
     ;               /* copy XXXX */
-    ;   }   }
+    ;       }
+    ;       IO_OUT->flags |= a_cmd->flags & PGOBLIN_PURGE;
+    ;   }
     ;   /**************************
     ;    *   CTL *
     ;    **************************/
@@ -1621,7 +1630,7 @@ pgoblin_execute( pgoblin_main  *options      /**/
     ;       ;   ;       sleep(exenv->options->flags & PGOBLIN_FORK_WAIT);
     ;       ;   ;   }
     ;       ;   ;   ex = pgoblin_cmdexe(cmd, exenv, rn);
-    ;       ;       ifBLIN_QX3("CHILD DONE %d", ex);
+    ;       ;       ifBLIN_QX3("CHILD DONE %d %d", ex, getpid());
     ;       ;       if  (jobflags & PGOBLIN_TOEXIT) exit(ex);
     ;       ;   } else {
                     pgoblin_jobl *j;
@@ -1695,7 +1704,7 @@ pgoblin_execute( pgoblin_main  *options      /**/
     ;   ;               ex = EX_IOERR;
     ;   ;               break;
     ;   ;           }
-    ;   ;           if  (IO_OUT->wri(IO_OUT->onu, cont, bl) < 0) {
+    ;   ;           if  (pgoblin_wr(IO_OUT, cont, bl) < 0) {
     ;   ;               ifBLIN_QW0("%s:%u unload mife write", finam, lfcount);
     ;   ;               ex = EX_IOERR;
     ;   ;               break;
@@ -1713,7 +1722,7 @@ pgoblin_execute( pgoblin_main  *options      /**/
     ;   ;       if  (!(IO_OUT->text)) {
     ;   ;           CICLEWARN(0, "unload no text");
     ;   ;       }
-    ;   ;       ex = IO_OUT->wri(IO_OUT->onu, IO_OUT->text, IO_OUT->length);
+    ;   ;       ex = pgoblin_wr(IO_OUT, IO_OUT->text, IO_OUT->length);
     ;   ;       if  (ex < 0) {
     ;   ;           ex = EX_IOERR;
     ;   ;           CICLEERR("unload text write");
@@ -1723,21 +1732,22 @@ pgoblin_execute( pgoblin_main  *options      /**/
     ;   ;       if  (ex) CICLEWARN(0, "IO_OUT->text not cleared");
     ;   ;   }
     ;   ;   if  (exenv->flags & PGOBLIN_FREEPQRE) {
-    ;   ;       if  (!(IO_OUT->pq)) {
-    ;   ;           CICLEWARN(0, "unload no pq");
-    ;   ;       }
-    ;   ;       ex = STYLS(pgoblin_load(options, 5, "0", -1))->table( exenv
-                                                                    , 0
-                                                                    , rn[PGO_COUT]
-                                                                    , rn[PGO_COUT]
-                                                                    , 0
-                                                                    , -2
-                                                                    );
-    ;   ;       if  (ex < 0) {
-    ;   ;           ex = EX_IOERR;
-    ;   ;           CICLEERR("unload pq");
-    ;   ;           continue;
-    ;   ;       }
+    ;   ;       if  (!(IO_OUT->flags & PGOBLIN_PURGE)) {
+    ;   ;           if  (!(IO_OUT->pq)) {
+    ;   ;               CICLEWARN(0, "unload no pq");
+    ;   ;           }
+    ;   ;           ex = STYLS(pgoblin_load(options, 5, "0", -1))->table( exenv
+                                                                        , 0
+                                                                        , rn[PGO_COUT]
+                                                                        , rn[PGO_COUT]
+                                                                        , 0
+                                                                        , -2
+                                                                        );
+    ;   ;           if  (ex < 0) {
+    ;   ;               ex = EX_IOERR;
+    ;   ;               CICLEERR("unload pq");
+    ;   ;               continue;
+    ;   ;       }   }
     ;   ;       ex = cleareg(exenv, RNOUT, PGOBLIN_PQRESULT);
     ;   ;       if  (ex) CICLEWARN(0, "IO_OUT->pq not cleared");
     ;   ;   }
