@@ -25,7 +25,7 @@
  */
 
 #ident "@(#) Copyright (C)2017..2022 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: func.c,v 1.66 2022/01/07 13:57:49 babolo Exp $"
+#ident "@(#) $Id: func.c,v 1.68 2022/03/26 23:07:14 babolo Exp $"
 
 #define BLIN_COMPAT      4
 #define Bpars_COMPAT     3
@@ -159,13 +159,13 @@ pgoblin_upio(pgoblin_main *options, pgoblin_nr nio) {               /**
         goto out;
     }
     if  (!((1ULL << nio) & options->useio)) {
-        ifBLIN_QX4("IO reg %u masked+", nio);
+        ifBLIN_QX4("IO reg %u masked+", pgoblin_regn[nio]);
         prev = options->io[nio];
         options->io[nio] = NULL;
     }
     if  (!(rio = options->io[nio])) {
         if  (!(options->io[nio] = rio = calloc(1, sizeof(pgoblin_rio)))) {
-            ifBLIN_QW0("No mem IO reg %u", nio);
+            ifBLIN_QW0("No mem IO %c", pgoblin_regn[nio]);
             goto out;
         }
         rio->prev = prev;
@@ -195,13 +195,13 @@ pgoblin_upcon(pgoblin_main *options, pgoblin_nr ndb)          {     /**
         goto out;
     }
     if  (!((1ULL << ndb) & options->usecon)) {
-        ifBLIN_QX4("CONN reg %u masked+", ndb);
+        ifBLIN_QX4("CONN %c masked+", pgoblin_regn[ndb]);
         prev = options->conn[ndb];
         options->conn[ndb] = NULL;
     }
     if  (!(rdb = options->conn[ndb])) {
         if  (!(options->conn[ndb] = rdb = calloc(1, sizeof(pgoblin_rdb)))) {
-            ifBLIN_QW0("No mem CONN reg %u", ndb);
+            ifBLIN_QW0("No mem CONN %c", pgoblin_regn[ndb]);
             goto out;
         }
         rdb->prev = prev;
@@ -230,13 +230,13 @@ pgoblin_upjob(pgoblin_main *options, pgoblin_nr njb) {              /**
         goto out;
     }
     if  (!((1ULL << njb) & options->usejob)) {
-        ifBLIN_QX4("JOB reg %u masked+", njb);
+        ifBLIN_QX4("JOB %c masked+", pgoblin_regn[njb]);
         prev = options->job[njb];
         options->job[njb] = NULL;
     }
     if  (!(rjb = options->job[njb])) {
         if  (!(options->job[njb] = rjb = calloc(1, sizeof(pgoblin_rjb)))) {
-            ifBLIN_QW0("No mem JOB reg %u", njb);
+            ifBLIN_QW0("No mem for JOB %c", pgoblin_regn[njb]);
             goto out;
         }
         rjb->prev = prev;
@@ -265,13 +265,13 @@ pgoblin_upsty(pgoblin_main *options, pgoblin_nr nst) {              /**
         goto out;
     }
     if  (!((1ULL << nst) & options->usesty)) {
-        ifBLIN_QX4("STY reg %u masked+", nst);
+        ifBLIN_QX4("STY %c masked+", pgoblin_regn[nst]);
         prev = options->style[nst];
         options->style[nst] = NULL;
     }
     if  (!(rst = options->style[nst])) {
         if  (!(options->style[nst] = rst = calloc(1, sizeof(pgoblin_rst)))) {
-            ifBLIN_QW0("No mem STY reg %u", nst);
+            ifBLIN_QW0("No mem for STY %c", pgoblin_regn[nst]);
             goto out;
         }
         rst->prev = prev;
@@ -302,7 +302,7 @@ pgoblin_meio(pgoblin_main *options, pgoblin_nr nio) {               /**
         goto out;
     }
     if  (!((1ULL << nio) & options->useio)) {
-        ifBLIN_QX4("IO reg %u masked", nio);
+        ifBLIN_QX4("IO %c masked", pgoblin_regn[nio]);
         prev = options->io[nio];
         errno = ENOENT;
         goto out;
@@ -334,7 +334,7 @@ pgoblin_mecon(pgoblin_main *options, pgoblin_nr ndb) {              /**
         goto out;
     }
     if  (!((1ULL << ndb) & options->usecon)) {
-        ifBLIN_QX4("CONN reg %u masked", ndb);
+        ifBLIN_QX4("CONN %c masked", pgoblin_regn[ndb]);
         prev = options->conn[ndb];
         errno = ENOENT;
         goto out;
@@ -365,7 +365,7 @@ pgoblin_mejob(pgoblin_main *options, pgoblin_nr njb) {              /**
         goto out;
     }
     if  (!((1ULL << njb) & options->usejob)) {
-        ifBLIN_QX4("JOB reg %u masked", njb);
+        ifBLIN_QX4("JOB %c masked", pgoblin_regn[njb]);
         prev = options->job[njb];
         errno = ENOENT;
         goto out;
@@ -396,7 +396,7 @@ pgoblin_mesty(pgoblin_main *options, pgoblin_nr nst) {              /**
         goto out;
     }
     if  (!((1ULL << nst) & options->usesty)) {
-        ifBLIN_QX4("STY reg %u masked", nst);
+        ifBLIN_QX4("STY %c masked", pgoblin_regn[nst]);
         prev = options->style[nst];
         errno = ENOENT;
         goto out;
@@ -430,7 +430,7 @@ pgoblin_pushio(pgoblin_main *options, pgoblin_nr nio) {             /**
     }
     prev = options->io[nio];
     if  (!(options->io[nio] = rio = calloc(1, sizeof(pgoblin_rio)))) {
-        ifBLIN_QW0("IO reg %u", nio);
+        ifBLIN_QW0("No mem for IO %c", pgoblin_regn[nio]);
         goto out;
     }
     rio->prev = prev;
@@ -450,7 +450,7 @@ pgoblin_rdb *
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_pushcon(pgoblin_main *options, pgoblin_nr ndb)          {     /**
+pgoblin_pushcon(pgoblin_main *options, pgoblin_nr ndb) {            /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
@@ -465,7 +465,7 @@ pgoblin_pushcon(pgoblin_main *options, pgoblin_nr ndb)          {     /**
     }
     prev = options->conn[ndb];
     if  (!(options->conn[ndb] = rdb = calloc(1, sizeof(pgoblin_rdb)))) {
-        ifBLIN_QW0("CONN reg %u", ndb);
+        ifBLIN_QW0("No mem for CONN %c", pgoblin_regn[ndb]);
         goto out;
     }
     rdb->prev = prev;
@@ -481,7 +481,7 @@ pgoblin_rjb *
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_pushjob(pgoblin_main *options, pgoblin_nr njb) {              /**
+pgoblin_pushjob(pgoblin_main *options, pgoblin_nr njb) {            /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
@@ -495,7 +495,7 @@ pgoblin_pushjob(pgoblin_main *options, pgoblin_nr njb) {              /**
     }
     prev = options->job[njb];
     if  (!(options->job[njb] = rjb = calloc(1, sizeof(pgoblin_rjb)))) {
-        ifBLIN_QW0("JOB reg %u", njb);
+        ifBLIN_QW0("No mem for JOB %c", pgoblin_regn[njb]);
         goto out;
     }
     rjb->prev = prev;
@@ -511,7 +511,7 @@ pgoblin_rst *
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_pushsty(pgoblin_main *options, pgoblin_nr nst) {              /**
+pgoblin_pushsty(pgoblin_main *options, pgoblin_nr nst) {            /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
@@ -525,7 +525,7 @@ pgoblin_pushsty(pgoblin_main *options, pgoblin_nr nst) {              /**
     }
     prev = options->style[nst];
     if  (!(options->style[nst] = rst = calloc(1, sizeof(pgoblin_rst)))) {
-        ifBLIN_QW0("STY reg %u", nst);
+        ifBLIN_QW0("No mem for STY %c", pgoblin_regn[nst]);
         goto out;
     }
     rst->prev = prev;
@@ -640,3 +640,39 @@ out:
     return(ex);
 #   undef blin_internal_flags
 }
+
+ssize_t
+/**********************************************************************
+ **********************************************************************
+ **                                                                  **/
+pgoblin_wr(pgoblin_rio *rio, const void *buf, size_t size) {        /**
+ **                                                                  **
+ **********************************************************************
+ **********************************************************************/
+#   define blin_internal_flags (rio->flags & BLIN_MASK)
+    ssize_t e;
+
+    if  (!(PGOBLIN_END & rio->flags)) {
+        if  (0 > (e = rio->wri(rio->onu, buf, size))) {
+            if  (!(PGOBLIN_TRAPDEF & rio->flags)) {
+                ifBLIN_QW0("IO out");
+            } else {
+                ifBLIN_QW0("IO out %s", (PGOBLIN_ERROR & rio->flags) ? "ignore" : "error");
+                e = 0;
+            }
+            if  (!(PGOBLIN_ERROR & rio->flags)) {
+                rio->flags |= PGOBLIN_ERROR;
+            } else {
+                rio->flags |= PGOBLIN_END;
+            }
+        } else {
+            rio->flags &= ~PGOBLIN_END;
+        }
+    } else if (!(PGOBLIN_TRAPDEF & rio->flags)) {
+        e = -1;
+    } else {
+        e = size;
+    }
+    return(e);
+#   undef blin_internal_flags
+}
