@@ -1,5 +1,5 @@
 /*-
- * Copyright (C)2002..2018 @BABOLO http://www.babolo.ru/
+ * Copyright (C)2002..2021 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,13 +24,14 @@
  * SUCH DAMAGE.
  */
 
-#ident "@(#) Copyright (C)2002..2018 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: parser.c,v 1.62 2018/07/25 16:34:11 babolo Exp $"
+#ident "@(#) Copyright (C)2002..2021 @BABOLO http://www.babolo.ru/"
+#ident "@(#) $Id: parser.c,v 1.69 2021/12/26 13:25:31 babolo Exp $"
 
-#define BLIN_COMPAT      3
+#define BLIN_COMPAT      4
 #define Bpars_COMPAT     3
-#define MIFE_COMPAT      4
-#define PGOBLIN_COMPAT   2
+#define MULAR_COMPAT     0
+#define MIFE_COMPAT      5
+#define PGOBLIN_COMPAT   4
 #define PGOBLIN_INTERNAL 1
 
 #include <sys/types.h>
@@ -46,6 +47,7 @@
 #include <err.h>
 #include <babolo/BLINflag.h>
 #include <babolo/parser.h>
+#include <multilar.h>
 #include <mife.h>
 #include "pgoblin.h"
 
@@ -148,16 +150,23 @@ placmd( pgoblin_main *options                                       /**/
 /**/  , u_int32_t     r_cmd                                         /**/
 /**/  , char         *r_beg                                         /**/
 /**/  , u_char       *r_end                                         /**/
-/**/  , int           lfcount                                       /**/
+/**/  , u_int32_t     lfcount                                       /**/
 /**/  , u_int32_t    *localflags                                    /**/
 /**/  ) {                                                           /**
  **                                                                  **
  **********************************************************************/
-    int ex = EX_OK;
-    int i;
+    pgoblin_tuple *command;
+    int            ex = EX_OK;
+    int            i;
 
 #   define blin_internal_flags (options->flags & BLIN_MASK)
-    ifBLIN_QX2("[%"BLIN_D"]=%02X(%016"BLIN_O"X)", pgm->curr, r_cmd, r_parm);
+    ifBLIN_QX3( "[%"BLIN_D"]=%03X=%s%.*s(%016"BLIN_O"X)"
+              , MULAR_NEXT(pgm->code)
+              , r_cmd
+              , pgoblin.a[r_cmd].str
+              , 9 - strlen(pgoblin.a[r_cmd].str), "         "
+              , r_parm
+              );
     if  (!r_cmd) {
         ifBLIN_QX0("Unknown command at line %d", lfcount);
         ERROUT(EX_DATAERR, EINVAL);
@@ -169,9 +178,6 @@ placmd( pgoblin_main *options                                       /**/
     if  (!(~pgoblin.a[r_cmd].flags & PGOBLIN_ILLEGAL)) {
         ifBLIN_QX0("Illegal command at line %d", lfcount);
         ERROUT(EX_SOFTWARE, EDOOFUS);
-    } else if (pgm->curr > pgm->maxx) {
-        ifBLIN_QX0("Programm too big");
-        ERROUT(EX_SOFTWARE, EFBIG);
     } else if (pgoblin.a[r_cmd].flags & PGOBLIN_IGNORE) {
         *localflags |= PGOBLIN_IGNORE;
     } else if (!(*localflags & PGOBLIN_IGNORE) || (r_cmd != PGOBLIN_cont)) {
@@ -189,22 +195,25 @@ placmd( pgoblin_main *options                                       /**/
             }
             r_parm >>= PGOBLIN_BPARG;
         }
-        if  (r_cmd & PGOBLIN_EXTENDED) {
-            pgm->code[pgm->curr].cmd = PGOBLIN_extended | (cc & ~PGOBLIN_COMMAND);
-            pgm->code[pgm->curr].ecmd = (r_cmd & PGOBLIN_COMMAND);
+        if  (!(command = mular_add(pgm->code))) {
+            ifBLIN_QX0("No room for programm");
+            ERROUT(EX_SOFTWARE, EFBIG);
+        } else if (r_cmd & PGOBLIN_EXTENDED) {
+            command->linum = lfcount;
+            command->cmd = PGOBLIN_extended | (cc & ~PGOBLIN_COMMAND);
+            command->ecmd = (r_cmd & PGOBLIN_COMMAND);
         } else {
-            pgm->code[pgm->curr].cmd = (r_cmd & PGOBLIN_COMMAND) | (cc & ~PGOBLIN_COMMAND);
-            pgm->code[pgm->curr].lit = NULL;
+            command->linum = lfcount;
+            command->cmd = (r_cmd & PGOBLIN_COMMAND) | (cc & ~PGOBLIN_COMMAND);
+            command->lit = NULL;
             if  (r_beg && r_end) {
                 *r_end = '\0';
                 if  (!(pgoblin.a[r_cmd].flags & PGOBLIN_NOLITER)) {
-                    pgm->code[pgm->curr].lit = r_beg;
+                    command->lit = r_beg;
                 }
-                if  (pgm->code[pgm->curr].lit) {
-                        ifBLIN_QX2(">%s~", pgm->code[pgm->curr].lit);
-        }   }   }
-        pgm->curr++;
-    }
+                if  (command->lit) {
+                        ifBLIN_QX3(">%s~", command->lit);
+    }   }   }   }
 out:
     return(ex);
 #   undef blin_internal_flags
@@ -225,17 +234,23 @@ int
 /********************************************************************************************
  ********************************************************************************************
  **                                                                                        **/
-pgoblin_parser(pgoblin_main *options, u_char *text, ssize_t txlen, pgoblin_prog **ppgm) { /**
+pgoblin_parser( pgoblin_main  *options                                                    /**/
+              , u_char        *text                                                       /**/
+              , ssize_t        txlen                                                      /**/
+              , pgoblin_prog **ppgm                                                       /**/
+              , u_int32_t      lfcount                                                    /**/
+              , const char    *finam                                                      /**/
+              ) {                                                                         /**
  **                                                                                        **
  ********************************************************************************************
  ********************************************************************************************/
+    size_t         sz[]    = {512, 512, 512};
     u_char        *literal = NULL;
-    char          *r_beg = NULL;
-    u_char        *r_end = NULL;
+    char          *r_beg   = NULL;
+    u_char        *r_end   = NULL;
     u_int32_t      localflags;
-    int            ex = EX_OK;
-    pgoblin_prog  *pgm = NULL;
-    int            lfcount;
+    int            ex      = EX_OK;
+    pgoblin_prog  *pgm     = NULL;
     u_int32_t      control;
     u_int64_t      r_parm;
     ssize_t        offset;
@@ -266,24 +281,29 @@ pgoblin_parser(pgoblin_main *options, u_char *text, ssize_t txlen, pgoblin_prog
             ifBLIN_QX0("Illegal pgoblin program");
             ERROUT(EX_DATAERR, EPROGMISMATCH);
         }
+    } else if (!(pgm = calloc(1, sizeof(pgoblin_prog)))) {
+        ifBLIN_QW0("Calloc failed for pgm");
+        ex = EX_OSERR;
+        goto out;
     } else {
-        if  (!(pgm = malloc(sizeof(pgoblin_tuple) * PGOBLIN_PROGSIZE + sizeof(pgoblin_prog)))) {
-            ifBLIN_QX0("Malloc failed in parser #1");
-            ERROUT(EX_OSERR, ENOMEM);
-        }
-        bzero(pgm, sizeof(pgoblin_tuple) * PGOBLIN_PROGSIZE + sizeof(pgoblin_prog));
+        /* pgm->debug = NULL; */
         strncpy(pgm->id, "#pGoblin-" VERS, PGOBLIN_STRING_ID_LEN);
-        pgm->maxx = PGOBLIN_PROGSIZE;
-        pgm->curr = 0;
-        pgm->debug = NULL;
-    }
+        pgm->code = mular_create( MULAR_HEXU | MULAR_STRI | MULAR_UPPE | MULAR_OBJ
+                                , 3
+                                , sizeof(pgoblin_tuple)
+                                , sz
+                                );
+        if  (!pgm->code) {
+            ifBLIN_QW0("mular_create failed for pgm->code");
+            ex = EX_OSERR;
+            goto out;
+    }   }
     if  (0 >= txlen) {
         ifBLIN_QX1("Empty pgoblin program text");
         goto out;
     }
-    ifBLIN_QX3("pgm@%"BLIN_D":%"BLIN_D, pgm->curr, pgm->maxx);
-    blin_stateheader(BLIN_4STA24G, pn);
-    lfcount = 0;
+    ifBLIN_QX3("pgm@%"BLIN_D, MULAR_NEXT(pgm->code));
+    blin_stateheader(BLIN_5STO24G, pn);
     qp = &s;
     qq = &qp;
     r_parm = 0;
@@ -302,7 +322,7 @@ pgoblin_parser(pgoblin_main *options, u_char *text, ssize_t txlen, pgoblin_prog
             s = 0;
         }
         control = automa[state][(u_char)p];
-        blin_statebody( BLIN_4STO24G
+        blin_statebody( BLIN_5STO24G
                       , pn
                       , statenames
                       , clasnames[(u_char)p]
