@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: pgoblin.h,v 1.193 2022/01/23 18:06:14 babolo Exp $
+ * $Id: pgoblin.h,v 1.195 2022/03/26 22:17:10 babolo Exp $
  */
 
 #ifndef _PGOBLIN_H_
@@ -82,13 +82,15 @@ __BEGIN_DECLS
 # define  PGOBLIN_ANYIVL    (PGOBLIN_PATHPARM | PGOBLIN_TEXTPARM | PGOBLIN_MIFEDES | PGOBLIN_PQRESULT)
 /**** First byte ends ****/
 # define  PGOBLIN_EXTENDED  0x000100  /*     -  +     * extended command                */
+# define  PGOBLIN_PURGE     0x000200  /*  +  -  +  +  * ignore Pq out                   */
 # define  PGOBLIN_FORKED    0x000800  /*     -  +  #  *                                 */
 # define  PGOBLIN_STYLEN    0x003000  /*     -  +     * phys addr of R_STYLE            */
-# define  PGOBLIN_TRAPDEF   0x001000  /*     -     #  *                                 */
-# define  PGOBLIN_ROLLBACK  0x002000  /*     -     #  *                                 */
-# define  PGOBLIN_END       0x004000  /*     -     #  *                                 */
-# define  PGOBLIN_ERROR     0x008000  /*     -     #  *                                 */
-# define  PGOBLIN_EXTABLE   0x00FF00  /*     -     #  *                                 */
+# define  PGOBLIN_TRAPDEF   0x001000  /*  +  -  +  #  *                                 */
+# define  PGOBLIN_ROLLBACK  0x002000  /*     -  +  #  *                                 */
+# define  PGOBLIN_END       0x004000  /*  +  -     #  *                                 */
+# define  PGOBLIN_ERROR     0x008000  /*  +  -     #  *                                 */
+# define  PGOBLIN_EXTABLE   0x00FD00  /*     -     #  *                                 */
+# define  PGOBLIN_OUTCLEAR  (PGOBLIN_TRAPDEF | PGOBLIN_END | PGOBLIN_ERROR)
 /**** Second byte ends ****/
 # define  PGOBLIN_FREEPATH  0x010000  /*  +  -     +  * free() Pf after using           */
 # define  PGOBLIN_FREETEXT  0x020000  /*  +  -     +  * free() Pt after using           */
@@ -774,6 +776,7 @@ __BEGIN_DECLS
    extern const char * const pgoblin_syntaxa_flags[];
 
    int            pgoblin_cmdexe __P((int cmd, pgoblin_exenv *exenv, pgoblin_nr *rn));
+   void            pgoblin_toexe __P((int cmd, pgoblin_exenv *exenv, pgoblin_nr *rn));
    int            pgoblin_listen __P((pgoblin_exenv *exenv, pgoblin_nr *rn));
    int             pgoblin_begin __P((pgoblin_exenv *exenv, pgoblin_nr *rn));
    int               pgoblin_end __P((pgoblin_exenv *exenv, pgoblin_nr *rn));
@@ -817,7 +820,6 @@ __BEGIN_DECLS
    int           pgoblin_getmeta __P((pgoblin_exenv *exenv, pgoblin_nr *rn));
    int              pgoblin_info __P((pgoblin_exenv *exenv, pgoblin_nr *rn));
    int            pgoblin_rollbk __P((pgoblin_exenv *exenv, pgoblin_nr *rn));
-   void            pgoblin_toexe __P((int cmd, pgoblin_exenv *exenv, pgoblin_nr rn[pgoblin.symparam]));
    int            pgoblin_wident __P((pgoblin_main *options, int num, char *entry));
    pgoblin_rio     *pgoblin_upio __P((pgoblin_main *options, pgoblin_nr nio));
    pgoblin_rdb    *pgoblin_upcon __P((pgoblin_main *options, pgoblin_nr ndb));
@@ -835,6 +837,7 @@ __BEGIN_DECLS
    int            pgoblin_popcon __P((pgoblin_main *options, pgoblin_nr ndb));
    int            pgoblin_popjob __P((pgoblin_main *options, pgoblin_nr njb));
    int            pgoblin_popsty __P((pgoblin_main *options, pgoblin_nr nst));
+    ssize_t           pgoblin_wr __P((pgoblin_rio *rio, const void *buf, size_t size));
    ssize_t       pgoblin_onecopy __P((char *escaped, const char *in, ssize_t l, int end));
    int           pgoblin_onecsym __P((char in));
    pgoblin_realpq  *pgoblin_db00 __P((BLIN_flag flags));
