@@ -25,7 +25,7 @@
  */
 
 #ident "@(#) Copyright (C)2008..2022 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: style_0.c,v 1.58 2022/01/08 18:35:13 babolo Exp $"
+#ident "@(#) $Id: style_0.c,v 1.59 2022/03/26 22:17:10 babolo Exp $"
 
 #define BLIN_COMPAT      4
 #define Bpars_COMPAT     3
@@ -117,7 +117,7 @@ sout(pgoblin_exenv *exenv, pgoblin_nr nou, char *c, ssize_t l, ssize_t ll) { /**
     if  (c) {
         ifBLIN_QX5("%"BLIN_U"<%"BLIN_U"=%.*s~", l, ll, (int)l, c);
         if  (rou->wri) {
-            ex = rou->wri(rou->onu, c, l);
+            ex = pgoblin_wr(rou, c, l);
             ifBLIN_QX5("%d", ex);
         } else if (0 > ll) {
             ifBLIN_QX0("invalid out buf ");
