@@ -1,5 +1,5 @@
 /*-
- * Copyright (C)2002..2018 @BABOLO http://www.babolo.ru/
+ * Copyright (C)2002..2021 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,13 +24,14 @@
  * SUCH DAMAGE.
  */
 
-#ident "@(#) Copyright (C)2002..2018 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: vdump.c,v 1.33 2018/07/16 23:50:40 babolo Exp $"
+#ident "@(#) Copyright (C)2002..2021 @BABOLO http://www.babolo.ru/"
+#ident "@(#) $Id: vdump.c,v 1.38 2021/10/24 13:58:58 babolo Exp $"
 
-#define BLIN_COMPAT      3
+#define BLIN_COMPAT      4
 #define Bpars_COMPAT     3
-#define MIFE_COMPAT      4
-#define PGOBLIN_COMPAT   2
+#define MULAR_COMPAT     0
+#define MIFE_COMPAT      5
+#define PGOBLIN_COMPAT   4
 #define PGOBLIN_INTERNAL 1
 
 #include <sys/types.h>
@@ -46,6 +47,7 @@
 #include <err.h>
 #include <babolo/BLINflag.h>
 #include <babolo/parser.h>
+#include <multilar.h>
 #include <mife.h>
 #include "pgoblin.h"
 
@@ -64,23 +66,33 @@ pgoblin_Vdump(pgoblin_main *options, pgoblin_prog *pgm) {           /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-    ssize_t p;
-    int i;
-    u_int32_t t, cmd;
-    char *lit;
+    pgoblin_tuple *command;
+    u_int32_t      cmd;
+    char          *lit;
+    size_t         p;
+    int            i;
+    u_int32_t      t;
 
 #   define blin_internal_flags (options->flags & BLIN_MASK)
-    if  (pgm) ifBLIN_QX3("+ pgm=%"BLIN_D, pgm->curr); else ifBLIN_QX3("+ pgm=NULL");
+    if  (pgm) {
+        ifBLIN_QX3("+ pgm=%"BLIN_D, MULAR_NEXT(pgm->code));
+    } else {
+        ifBLIN_QX3("+ pgm=NULL");
+    }
     printf("+######################################");
     for (p = 0; p < PGOBLIN_STRING_ID_LEN && pgm->id[p]; p++) printf("%c", pgm->id[p]);
-    for (p = 0; p < pgm->curr; p++) {
-        t = pgm->code[p].cmd;
+    for (p = 0; p < MULAR_NEXT(pgm->code); p++) {
+        if  (!(command = mular_getix(pgm->code, p))) {
+            ifBLIN_QW0("mular_getix %d", p);
+            goto out;
+        }
+        t = command->cmd;
         cmd = t & PGOBLIN_COMMAND;
         if  (cmd == PGOBLIN_extended) {
-            cmd = (pgm->code[p].ecmd & PGOBLIN_COMMAND) | PGOBLIN_EXTENDED;
+            cmd = (command->ecmd & PGOBLIN_COMMAND) | PGOBLIN_EXTENDED;
             lit = NULL;
         } else {
-            lit = pgm->code[p].lit;
+            lit = command->lit;
         }
         if  (cmd >= pgoblin.max && !(~pgoblin.a[cmd].flags & PGOBLIN_ILLEGAL)) {
             printf("\n#ILLEGAL ");
@@ -91,8 +103,9 @@ pgoblin_Vdump(pgoblin_main *options, pgoblin_prog *pgm) {           /**
         for (i = 0; i < pgoblin.binparam; i++)
             printf("%c", pgoblin_regn[(t >>= PGOBLIN_BPARG) & PGOBLIN_ARGMASK]);
         printf(" ([%d]=%03X)", (int)p, cmd);
-        if  (lit) printf("\n%s", pgm->code[p].lit);
+        if  (lit) printf("\n%s", command->lit);
     }
+out:
     printf("\n-######################################\n");
     ifBLIN_QX3("-");
 #   undef blin_internal_flags
